; TODO
;
; - Game start / Attract mode
; - Death
; - Level editor
; - 50/60hz switchable (for PAL)
; - Music/SFX switchable?
;
; - Block for faster movement
; - Disappearing blocks? (with countdown every pass decreases?)
; - Block that forces movement in the direction you're going
; - Animate falling blocks?
; - Adjust speed for x and y ball movement (should be the same)
; - Pack level header
; * DEMO Music
; * New DEMO level
; * Music/No Music switch
; * FOR DEMO ONLY: Select a random level with reset
; * Control music:
;   * stop
;   * start
;   * skip to pattern/possibly change song
; * Music
; * Counters/indicators:
;   * tiles left (2 numbers)
;   * level
;   * lifes (dots)
; * Navigation on memory blocks map (BallPos)
; * ADC 6 and SBC 5 to BallPos?!?!
; * Make the "level color" byte a pointer (low byte only) to a level color table
; * Pack/Unpack levels
; * Use JMPs instead of JSRs in the Mail Loop
; * Check that INTIM loops never happen across a page
; * Merge VerticalBlank + CheckSwitches + GameCalc
; * Do overscan with Timer
; * Remove 1st byte of each level (not required anymore)
; * Levels must be all the same size
; * Remove garbage at the bottom
; * Remove 1 cycle rom NextRow
; * Get back level data into ROM!!! (since it's impossible to pack it, anyway)
; * Copy level data to RAM. This will allow to:
;   - pack level data in ROM (two tiles per byte)
;   - animate tiles
;   - easier (=faster) addressing

	processor 6502
	org $F000

	include "..\gen\vcs.h"
	include "..\gen\macros.asm"


PAL_COLORS = 0
PAL_60hz = 0

BALLCOLOR = $4a
BGCOLOR = $94
TITLECOLOR = $f2
TILESLEFTCOLOR = $e2

BALL_HEIGHT = 4

BALLANIMFRAMES = 16	;(0..)

BALLANIMDELAY_STANDARD = 4	; Animation speed delay (lower = faster)
BALLMOVEDELAY_STANDARD = 8	; Movement speed delay (lower = faster)

; For faster movement
BALLANIMDELAY_FAST = 4-1	; Animation speed delay (lower = faster)
BALLMOVEDELAY_FAST = 8-2	; Movement speed delay (lower = faster)

; For double jump
BALLANIMDELAY_BIGJUMP = 4+2	; Animation speed delay (lower = faster)
BALLMOVEDELAY_BIGJUMP = 8-2	; Movement speed delay (lower = faster)

LEVELRAM_LASTBYTE = 71+6	; Pointer to the last byte of level ram
;LEVELSIZE_ROM = 76		; Size of each level in ROM in bytes (including header)
LEVELSIZE_ROM = 40		; Size of each level in ROM in bytes (including header)
LEVEL_HEADERLASTBYTE = 4	; Pointer to the last byte of the level header in ROM

BLOCK_BLANK = $00 *8
BLOCK_NORMAL = $01 *8
BLOCK_IRON = $02 *8
BLOCK_RIGHT = $03 *8
BLOCK_LEFT = $04 *8
BLOCK_DOWN = $05 *8
BLOCK_UP = $06 *8
BLOCK_JUMP = $07 *8

RAND_SEED = $38		; Initial random seed

PtrLo = $80
PtrHi = $81
PS_temp = $8e	; reuse of LoopCount (PS_temp is only used outside of game kernel)
Temp = $81	; reuse of PtrHi
Temp2 = $82	; Reuse of ptr0
Temp3 = $83	; Reuse of ptr0
LevelColor = $80	; Reuse of PtrLo

BallAnimDelayStart = $82	; Reuse of ptr0
BallMoveDelayStart = $83	; Reuse of ptr0

ptr0 = $82	; 2-bytes pointers to sprites data
ptr1 = $84
ptr2 = $86
ptr3 = $88
ptr4 = $8a
ptr5 = $8c

LoopCount = $8e
ScanCount = $8f		; Just needed for SkipDraw
ScanNumber = $90
BallTabPtr = $91
RowsNumber = $92
BallX = $93
BallY = $94
BallPos = $95		; Ball's position on blocks map in RAM
BallMoveDelay = $96
BallAnimDelay = $97
BallAnimPtr = $98

LevelDataPtr = $99

LevelRam = $9a	; 78 bytes
; Free from $e8 (included)
LivesLeft = $e8

; --- Song Player variables ---
tempSong = $81	; Reuse of Temp
temp16L = $e9
temp16H = $ea
;temp16L = $84	; Reuse of ptr1
;temp16H = $85	; Reuse of ptr1
tempoCount = $eb
atten = $ec
;atten = $82	; Reuse of Temp2
beat = $ed
measure = $ee	; Change this to skip throu song

random = $ef	;

MapX = $f0	; Player X and Y positions and pointer on the map
MapY = $f1

BaseBCD = $f2		; BCD bytes start here
TilesLeft = $f2
CurrentLevel = $f3
BigJump = $f4		; Flag to know the ball is making a double jump

Direction = $f5
OldJoystick0 = $f6
Joystick0 = $f7
MoveCount = $f8

; -- end of ram at $fb included --- (nesting 2 JSRs)

; --------- Game entry point is here ----------
Start
	SEI  ; Disable interrupts, if there are any.
	CLD  ; Clear BCD math bit.
	LDX  #$FF
	TXS  ; Set stack to beginning ($FF).

	LDA #0
MainClearRamLoop
	STA 0,X
	DEX
	BNE MainClearRamLoop	; Clear RAM and 0-page registers

; Enable these lines to start with a random level (random only required for DEMO release)
	LDA #RAND_SEED	; demo
	STA random	; demo

	JSR InitRandomLevel	; demo

;	LDA #$04	; final
	STA CurrentLevel	; final
	JSR InitGame

MainLoop
	JMP  VerticalBlank ; 3 lines VSYNC + VBLANK + Checkswitches + GameCalc
MainDrawScreen
	JMP  DrawScreen    ; Draw the screen (once VBLANK has finished)
MainOverscan
	JMP  OverScan      ; Overscan + jump back to MainLoop

; ----------

VerticalBlank	subroutine ; *** VERTICAL BLANK HANDLER
	LDX  #0
	LDA  #2
	STA  WSYNC
	STA  VSYNC ;Begin vertical sync.
	STA  WSYNC ; First line of VSYNC
	STA  WSYNC ; Second line of VSYNC.

	IF PAL_60hz
	LDA  #52	; 37 vblank lines (45 if PAL) with timer + 3 lines with WSYNCs
	ELSE
	LDA  #44	; 37 vblank lines (45 if PAL) with timer + 3 lines with WSYNCs
	ENDIF
	STA  TIM64T

	LDA #0
	STA CXCLR

	STA  WSYNC ; Third line of VSYNC.
	STA  VSYNC ; (0) End VSYNC

;	RTS  

;CheckSwitches	subroutine ; *** CONSOLE SWITCH HANDLER
	LDA #0
	STA COLUBK  ; Background will be black.
	LDA SWCHA       ;Read joystick 0
	STA Joystick0   ;Store for later use
;	CMP OldJoystick0
;	BEQ .NoUpdateOldJoystick0
	AND #$f0
	CMP #$f0
	BEQ .NoUpdateOldJoystick0
	STA OldJoystick0
.NoUpdateOldJoystick0

;	LDA SWCHB
;	AND #%00000001
;	BNE NoResetGame
	;JSR GameInit	; Se RESET e' premuto, reinizializza il gioco
	;JSR NewLevel

	LDA SWCHB
	LSR
	BCS .NoResetGame	; Reset not pressed? jump
	JSR InitRandomLevel	; Only needed for DEMO release
	JSR InitGame
.NoResetGame
;	RTS

;GameCalc	subroutine ;*** GAME CALCULATION ROUTINES
; -- Ball animation
	LDA SWCHB	; Left difficulty switch toggles music
	AND #P1_Diff
	BEQ .PlayMusic
	LDA #0
	STA AUDV0
	STA AUDV1
	JMP .ContinueMusicCheck
.PlayMusic
	JSR songPlayer
.ContinueMusicCheck

	LDY BallAnimPtr
	LDA BallAnimTab,Y
	STA BallTabPtr		; Set animation pointer to animation data tables
	DEC BallAnimDelay
	BNE .NoZeroBallAnimDelay
	LDA BigJump
	BNE .BlockJump
	LDA #BALLANIMDELAY_STANDARD
	.byte $2c
.BlockJump
	LDA #BALLANIMDELAY_BIGJUMP
	STA BallAnimDelay	; Reset ball animation delay and go to the next frame of animation
	DEC BallAnimPtr		; Next frame of animation
	DEY
	LDA BallAnimTab,Y	; Get next frame
	BNE .NoZeroBallAnimPtr	; If it's 0, reset anim
	LDA #BALLANIMFRAMES	; Make animation wrap!
	STA BallAnimPtr

.NoZeroBallAnimPtr
.NoZeroBallAnimDelay	; No change of animation frame

; -- Read joystick and move ball
; Make it load a backup of Joystick0 that only updates if Joystick0 changes
;	DEC MoveCount
;	BPL .MoveBall		; Ball still moving, don't change direction
	LDA BallAnimTab,Y
;	BNE .MoveBall
	BEQ .CheckBlocks
	JMP .MoveBall
; ---------------------------------------------------
; ------ Check current tile and prepare action ------
; ---------------------------------------------------
.CheckBlocks
	LDA BallX	; Check for ball out of bounds on the right
	CMP #110
	BCC .ContBlockA
	JSR InitGame	; DEATH
	JMP .EndMove
.ContBlockA
	LDA BallX	; Check for ball out of bounds on the left
	CMP #63
	BCS .ContBlockB
	JSR InitGame	; DEATH
	JMP .EndMove
.ContBlockB
	LDA BallY	; Check for ball out of bounds on top
	CMP #187
	BCC .ContBlockC
	JSR InitGame	; DEATH
	JMP .EndMove
.ContBlockC
	LDA BallY	; Check for ball out of bounds on top
	CMP #130
	BCS .ContBlockD
	JSR InitGame	; DEATH
	JMP .EndMove
.ContBlockD
	LDA #0
	STA BigJump
	LDX BallPos
	LDA LevelRam,X
	CMP #BLOCK_BLANK
	BNE .ContBlock1
	; Put here action
	; DEATH: reset level here
	JSR InitGame
	JMP .EndMove
.ContBlock1
	CMP #BLOCK_NORMAL
	BNE .ContBlock2
	; Put here action
	LDA #BLOCK_BLANK
	STA LevelRam,X
;	SEC	; Carry must be set here!
	SED
	LDA TilesLeft
	SBC #1
	STA TilesLeft
	;DEC TilesLeft
	CLD
	BNE .EndBlockCheck
	JSR InitRandomLevel
	JSR InitGame
	JMP .EndMove
.ContBlock2
	CMP #BLOCK_IRON
	BNE .ContBlock3
	; Put here action
	JMP .EndBlockCheck
.ContBlock3
	CMP #BLOCK_RIGHT
	BNE .ContBlock4
	; Put here action
	LDA #%01111111		; Force movement
	STA OldJoystick0
	LDA #BLOCK_BLANK
	STA LevelRam,X
	JMP .EndBlockCheck
.ContBlock4
	CMP #BLOCK_LEFT
	BNE .ContBlock5
	; Put here action
	LDA #%10111111		; Force movement
	STA OldJoystick0
	LDA #BLOCK_BLANK
	STA LevelRam,X
	JMP .EndBlockCheck
.ContBlock5
	CMP #BLOCK_DOWN
	BNE .ContBlock6
	; Put here action
	LDA #%11011111		; Force movement
	STA OldJoystick0
	LDA #BLOCK_BLANK
	STA LevelRam,X
	JMP .EndBlockCheck
.ContBlock6
	CMP #BLOCK_UP
	BNE .ContBlock7
	; Put here action
	LDA #%11101111		; Force movement
	STA OldJoystick0
	LDA #BLOCK_BLANK
	STA LevelRam,X
	JMP .EndBlockCheck
.ContBlock7
	CMP #BLOCK_JUMP
	BNE .ContBlock8
	; Put here action
	;LDA #BLOCK_JUMP
	LDA #1
	STA BigJump
	LDA #BLOCK_BLANK
	STA LevelRam,X
	JMP .EndBlockCheck
.ContBlock8

.EndBlockCheck
; -------- End of check for current tile ------------

	LDA OldJoystick0	; Time to change direction: read joystick buffer
	BPL .JoyRight
	ROL
	BPL .JoyLeft
	ROL
	BPL .JoyDown
	ROL
	BPL .JoyUp
	JMP .MoveBall

.JoyRight
	LDA #1
	STA Direction
	DEC BallPos
	LDA BigJump
	BEQ .NoBigJump1
	DEC BallPos
.NoBigJump1
	JMP .MoveBall
.JoyLeft
	LDA #2
	STA Direction
	INC BallPos
	LDA BigJump
	BEQ .NoBigJump2
	INC BallPos
.NoBigJump2
	JMP .MoveBall
.JoyDown
	LDA #3
	STA Direction
	LDA BallPos
	; Carry = 0
;	CLC
	SBC #6
	LDX BigJump
	BEQ .NoBigJump3
	SBC #6
.NoBigJump3
	STA BallPos
	LDA #27	; Make animation wrap!
	STA BallAnimPtr
	JMP .MoveBall
.JoyUp
	LDA #4
	STA Direction
	LDA BallPos
	; Carry = 1
;	CLC
	ADC #5
	LDX BigJump
	BEQ .NoBigJump4
	ADC #6	; Carry = 0
.NoBigJump4
	STA BallPos
	LDA #27	; Make animation wrap!
	STA BallAnimPtr
;	JMP .MoveBall

.MoveBall	; Move ball depending on the Direction byte
	DEC BallMoveDelay
	BNE .EndMove
	LDA BigJump
	BNE .BlockJump2
	LDA #BALLMOVEDELAY_STANDARD
	.byte $2c
.BlockJump2
	LDA #BALLMOVEDELAY_BIGJUMP
	STA BallMoveDelay

	LDA Direction
	CMP #1
	BEQ .MoveRight
	CMP #2
	BEQ .MoveLeft
	CMP #3
	BEQ .MoveDown
	CMP #4
	BEQ .MoveUp
	JMP .EndMove

.MoveRight
	INC BallX
	JMP .EndMove

.MoveLeft
	DEC BallX
	JMP .EndMove

.MoveDown
	DEC BallY
	JMP .EndMove

.MoveUp
	INC BallY
	JMP .EndMove

.EndMove
; --

; -- Prepare stuff for the next frame...
	LDA #191
	STA ScanCount

;	LDA LevelRowsNumber
;	LDA #12
	LDA #14
	STA RowsNumber

	;LDA #38
;	LDA #71
;	STA LevelDataPtr

;	LDA #2
;	STA LevelDataFlipFlop
; --
	JMP MainDrawScreen
;	RTS

OverScan	subroutine   ;*** OVERSCAN CALCULATIONS
; -- First, do vblank and clear some stuff
	IF PAL_60hz
	LDA  #37
	ELSE
	LDA  #37
	ENDIF
	STA  TIM64T
	;
	LDY #0
	STY PF0
	STY PF1
	STY PF1
	STY GRP0
	STY GRP1
	STY ENAM0
	STY ENAM1
	STY ENABL

;	IF PAL_60hz
;	LDX #36		; PAL
;	ELSE
;	LDX #30		; NTSC
;	ENDIF

;.KillLines
;	STA WSYNC
;	DEX
;	BNE .KillLines
	JSR WaitINTIM
	STA WSYNC

	JMP MainLoop
;	RTS

; Input: X: sprite's X position
CalcSprXPos	subroutine
	TXA	; absolute X pixel
	lsr
	lsr
	lsr
	lsr
	tay	; /16
	
	sta PS_temp
	
	;lda spr0x
	TXA
	and #15
	clc
	adc PS_temp
	cmp #15
	
	tya
	adc #0
	tay
	sty PS_temp
	iny
	iny
	iny
	
	;lda spr0x
	TXA
	and #15
	clc
	adc PS_temp	; calculate the remainder for the /15
	; Use remainder for fine adjustment
	eor #15
	sbc #7	; -8 to +7
	asl
	asl
	asl
	asl

;	RETURN:
;	A: Fine Movement (offset)
;	Y: Position
	RTS

CopyLevelToRam	subroutine	; Unpack and copy level data from ROM to RAM
;	LDA #1	; Current level
	LDA CurrentLevel
	ASL
	TAX
	DEX
	LDA LevelPointers,X
	STA PtrLo
;	STA LevelPtrLo
	DEX
	LDA LevelPointers,X
	STA PtrHi
;	STA LevelPtrHi

	LDX #LEVELRAM_LASTBYTE			; Point to the last byte of Level Ram
;	LDX #5
	LDY #5
	LDA #0
.ClearLevelRam			; Clear level Ram (last 6 bytes)
	STA LevelRam,X
	DEX
	DEY
	BPL .ClearLevelRam

	LDY #LEVEL_HEADERLASTBYTE
	LDA (PtrLo),Y
	STA BallY
	DEY
	LDA (PtrLo),Y
	STA BallX
	DEY
	LDA (PtrLo),Y
	;STA MapPtr		; Starting position
	STA BallPos
	DEY
	SED
	LDA (PtrLo),Y
	STA TilesLeft		; Tiles to destroy (BCD)
	CLD
	DEY
	LDA (PtrLo),Y		; Level Color offset
	STA Temp3
;	DEY
	LDY #LEVELSIZE_ROM			; Point to the last byte of level in rom

	LDX #LEVELRAM_LASTBYTE-6		; Point to the last row of Level Ram
;
.CopyLoop	; Copy and unpack from rom to ram
	LDA (PtrLo),Y
	STA Temp2
	AND #%00001111		; Get lower nibble
	ASL			; *2
	ASL			; *4
	ASL			; *8
	STA LevelRam,X
	DEX
	LDA Temp2
	AND #%11110000		; Get higher nibble
	LSR			; /2 (= byte*8)
	STA LevelRam,X
	DEX
	DEY
	CPY #LEVEL_HEADERLASTBYTE
	BNE .CopyLoop

;	LDA <#TilesColor
;	STA PtrLo
;	CLC
;	ADC Temp3
;	LDA >#TilesColor
;	STA PtrHi
	LDA Temp3
	STA LevelColor

;	LDA (PtrLo),Y
;	STA LevelRam,X
;	DEX
;	DEY
;	CPY #LEVEL_HEADERLASTBYTE
;	BNE .CopyLoop

	RTS

InitGame	subroutine
	LDA #0
	STA Direction
	STA BigJump

;	LDA #$01
;	STA CurrentLevel

	LDA #3
	STA LivesLeft

	LDA #8*BALLMOVEDELAY_STANDARD -1
	STA MoveCount

	LDA #$f0
	STA OldJoystick0

	LDA #1
	STA ScanNumber

	LDA  #5		; Playfield reflected and with priority over sprites
;	LDA  #53		; Playfield reflected and qith priority over sprites
	STA  CTRLPF

	; X limits:
	; (55)63 - 103(111)
	;LDA #55
	;STA BallX
	;LDA #191
	;STA BallY

	LDA #3
	STA BallTabPtr

	LDA #BALLANIMDELAY_STANDARD
	STA BallAnimDelay

	LDA #BALLMOVEDELAY_STANDARD
	STA BallMoveDelay

	LDA #BALLANIMFRAMES
	STA BallAnimPtr

;	LDA #71
;	STA LevelDataPtr

	JSR CopyLevelToRam
	; Now we have start values in BallX and BallY, calculate BallPos
;	LDA BallY
;	ASL	; *2
;	ASL	; *4
;	CLC
;	ADC BallY
;	CLC
;	ADC BallY	; *6
;	CLC
;	ADC BallX
;	STA BallPos
;	LDA #77		; force 77 for testing purposes
;	STA BallPos

	;LDA #8
	;STA LevelRam+71

;	JSR InitScreenSprites
;	LDA #1
;	STA Direction

	RTS

DrawScreen	subroutine
;	LDA INTIM
;	BNE DrawScreen ; Whew!
	JSR WaitINTIM
	STA WSYNC
	STA VBLANK  ;End the VBLANK period with a zero.

	LDA #LEVELRAM_LASTBYTE
;	LDA #71+12
	STA LevelDataPtr

	STA HMCLR
	JSR PositionBall
	STA WSYNC
	STA HMOVE
	JSR Position6Sprites
	STA HMCLR		; Very important...!
	STA WSYNC
	STA HMOVE
;	JSR InitScreenSprites
;	STA WSYNC
;	LDA #BGCOLOR
	LDA #TITLECOLOR
	STA COLUP0
	STA COLUP1
	LDA #BALLCOLOR
	STA COLUPF

; Draw title
	LDA #%00000001
	STA PF1
	LDA #$ff
	STA PF2
;	LDX #9
;.LoopTitle
	LDA #$1
	STA CTRLPF
;	STA WSYNC
;	DEX
;	BPL .LoopTitle

	JSR InitLogoSprites

	LDA #7
	JSR Display6Sprite
	STA WSYNC
	JSR InitScreenSprites
;	STA WSYNC

	LDA #0
	STA PF2
	
	LDA #7
	JSR DrawPlayfield

;	LDA #%00000000
;	STA PF1
;	STA PF2

	DEC ScanNumber
	BPL .NoResetScanNumber
	INC ScanNumber
	INC ScanNumber
.NoResetScanNumber

; ----- Draw footer -----
;	STA WSYNC
	LDA #$ff
	STA PF2
	LDA #TILESLEFTCOLOR
	STA COLUP0
	STA COLUP1

; ----- Display remaining tiles ------
	LDA #$1
	STA CTRLPF

	lda #<LevelSprite	; Load level icon for level indicator
	sta ptr3
	lda #>LevelSprite
	sta ptr3+1

	lda #<TilesLeftSprite	; Load tiles left icon for tiles left indicator
	sta ptr0
	lda #>TilesLeftSprite
	sta ptr0+1

	ldx #0
;	lda #1
;	sta Temp
	ldy #4
	JSR InitFooterSprites	; Load pointers for remaining tiles indicator
	ldx #1
;	lda #1
;	sta Temp
	ldy #10
	JSR InitFooterSprites	; Load pointers for current level number indicator
	LDA #7
	JSR Display6Sprite
	
	LDX #4
	JSR EatLines		; A few more footer blank lines
	LDA #0
	STA PF2
	STA PF1

	LDA #BALLCOLOR
	STA COLUP0
	STA COLUP1
; ----- Display remaining lifes -----
	LDY #0
	LDX LivesLeft
	DEX
.loopLives
	LDA #<LifeSprite
	STA ptr0,y
	LDA #>LifeSprite
	STA ptr0+1,y
	INY
	INY
	DEX
	BPL .loopLives
	LDA #6
	SBC LivesLeft
	TAX
.loopLives2
	LDA #<BgTiles0
	STA ptr0,y
	LDA #>BgTiles0
	STA ptr0+1,y
	INY
	INY
	DEX
	BPL .loopLives2

	LDA #3
	JSR Display6Sprite

	LDX #4
	JSR EatLines		; Finish frame

	JMP MainOverscan
;	RTS

; This sub may not be required if 1st row is always blank
InitScreenSprites	subroutine
;	JSR Position6Sprites

; Load pointers to sprites data
; --- Load pointers with next row of level data
;	LDX #0
;	LDY LevelDataPtr	; 3
;.LoopLoadBg
;	LDA LevelData,Y	; 4
;	STA ptr0,X	; 3
;	DEY	; 2
;	INX
;	INX
;	CPX #12
;	BNE .LoopLoadBg
;	STY LevelDataPtr

;	LDX LevelDataPtr	; 3
;	LDX #71		; 2
	LDX LevelDataPtr
; Unrolling loop to save some cycles...
;	LDA (LevelPtrLo),Y	; 5
	LDA LevelRam,X	; 4
	STA ptr0	; 3
	DEX	; 2
;	LDA (LevelPtrLo),Y	; 5
	LDA LevelRam,X	; 4
	STA ptr1	; 3
	DEX	; 2
;	LDA (LevelPtrLo),Y	; 5
	LDA LevelRam,X	; 4
	STA ptr2	; 3
	DEX	; 2
;	LDA (LevelPtrLo),Y	; 5
	LDA LevelRam,X	; 4
	STA ptr3	; 3
	DEX	; 2
;	LDA (LevelPtrLo),Y	; 5
	LDA LevelRam,X	; 4
	STA ptr4	; 3
	DEX	; 2
;	LDA (LevelPtrLo),Y	; 5
	LDA LevelRam,X	; 4
	STA ptr5	; 3
	DEX	; 2
	STX LevelDataPtr	; 3
; ---

; Make a loop of the following stuff to save up rom space
;	LDA #<BgTiles0
;	STA ptr0
	LDA #>BgTiles0
	STA ptr0+1
;	LDA #<BgTiles1
;	STA ptr1
;	LDA #>BgTiles1
	STA ptr1+1
;	LDA #<BgTiles2
;	STA ptr2
;	LDA #>BgTiles2
	STA ptr2+1
;	LDA #<BgTiles3
;	STA ptr3
;	LDA #>BgTiles3
	STA ptr3+1
;	LDA #<BgTiles4
;	STA ptr4
;	LDA #>BgTiles4
	STA ptr4+1
;	LDA #<BgTiles5
;	STA ptr5
;	LDA #>BgTiles5
	STA ptr5+1	; 60 cycles

	RTS

InitLogoSprites	subroutine
	LDA #<LogoSpr0
	STA ptr0
	LDA #>LogoSpr0
	STA ptr0+1
	LDA #<LogoSpr1
	STA ptr1
	LDA #>LogoSpr1
	STA ptr1+1
	LDA #<LogoSpr2
	STA ptr2
	LDA #>LogoSpr2
	STA ptr2+1
	LDA #<LogoSpr3
	STA ptr3
	LDA #>LogoSpr3
	STA ptr3+1
	LDA #<LogoSpr4
	STA ptr4
	LDA #>LogoSpr4
	STA ptr4+1
	LDA #<LogoSpr5
	STA ptr5
	LDA #>LogoSpr5
	STA ptr5+1
	RTS

; Generic BCD to Pointers routine.
; Y: last pointer *2 (10 if 6 digits)
; X: offset from "BaseBCD" to the bytes to convert
InitFooterSprites	subroutine
;	lda #<BgTiles0
;	sta ptr4
;	sta ptr5
;	lda #>BgTiles0
;	sta ptr4+1
;	sta ptr5+1
	lda #1
	sta Temp

	lda BaseBCD,x
	and #$0f	; get LS nibble
	asl 
	asl 
	asl		; *8
.loop1
	adc #<Number0	; load pointer using nibble
	sta ptr0,y
	lda #0
	adc #>Number0
	sta ptr0+1,y
	dey
	dey
	dec Temp
	bmi .endloop1	; end loop if y < 0

	lda BaseBCD,x
	and #$f0	; get MS nibble
	lsr		; *8
	jmp .loop1
.endloop1
	RTS

	include "songplay.h"
	include "bouncedemo2.asm"
;	include "demosong.h"
	;include "testsong.asm"

	org $fa00

Display6Sprite	subroutine
	STA     LoopCount	; 3	(40 cycles)
.X1:
	STA     WSYNC

; 62 cycles
	LDY #10		; 2
.DelayLoop1
	DEY		; 2 * 11
	BPL .DelayLoop1	; 3 * 10 + 2 =  cycles
	BIT $00
	BIT $00

; 9
.X3a:
	NOP
	NOP
	NOP
	LDY     LoopCount	; 3
	LDA     (ptr0),Y
	STA     GRP0
	LDA     (ptr1),Y
	STA     GRP1
	LDA     (ptr2),Y
	STA     GRP0
	LDA     (ptr5),Y
	STA     Temp
	LDA     (ptr4),Y
	TAX
	LDA     (ptr3),Y
	LDY     Temp
	STA     GRP1
	STX     GRP0
	STY     GRP1
	STA     GRP0
	DEC     LoopCount
	BPL     .X3a

	LDA     #0
	STA     GRP0
	STA     GRP1
	STA     GRP0
	STA     GRP1
	RTS

Position6Sprites	subroutine
	LDA     #$01
	STA     VDELP0
	STA     VDELP1
	LDA     #$03
	STA     NUSIZ0
	STA     NUSIZ1
	LDA     #$46
	STA     COLUP0
	STA     COLUP1

	LDA #0
	STA COLUBK
	STA COLUPF
	STA GRP0
	STA GRP1
	STA REFP0
	STA REFP1
	STA ENAM0
	STA ENAM1
	STA ENABL
;	STA RESP0
;	STA RESP1
	STA HMCLR

	STA     WSYNC

;	NOP
;	NOP
;	NOP
;	NOP
;	NOP
;	NOP
;	NOP
;	NOP
;	NOP
;	NOP	; 20 cycles
;	NOP
;	NOP

;	NOP
;	NOP
;	NOP
;	NOP
;	NOP

;	NOP
;	NOP
;	NOP	; 40 cycles total

;	LDY #6		; 2
;.DelayLoop_introscreensprites1
;	DEY		; 2 * 7
;	BPL .DelayLoop_introscreensprites1	; 3 * 6 + 2 =  cycles
;	NOP
;	NOP
;	SLEEP 40
	SLEEP 38

	STA RESP0	; Position sprites on the screen
	STA WSYNC

;	NOP
;	NOP
;	NOP
;	NOP
;	NOP
;	NOP
;	NOP
;	NOP
;	bit $00
;	NOP	; 19 cycles
;	NOP
;	NOP

;	NOP
;	NOP
;	NOP
;	NOP
;	NOP

;	NOP
;	NOP
;	NOP	; 41 cycles

;	LDY #7		; 2
;.DelayLoop_introscreensprites2
;	DEY		; 2 * 7
;	BPL .DelayLoop_introscreensprites2	; 3 * 6 + 2 =  cycles

;	SLEEP 41
	SLEEP 39

	STA     RESP1

;	LDA     #$50    ; .. and shift them
;	LDA     HMP1
;	LDA     #$40
;	STA     HMP0

	;LDA     #$90    ; .. and shift them
	;LDA     HMP1
	LDA #%11110000
	STA HMP1
;	LDA     #$50
	LDA #%01000000
	STA     HMP0

	STA     WSYNC
	STA     HMOVE
	STA     WSYNC
	RTS

	org $fb00

PositionBall	subroutine
	LDX BallX
	JSR CalcSprXPos
	STA HMBL
	TYA
	TAX

	;LDX BallPos
	STA WSYNC
.BallXposLoop1
	DEX
	BPL .BallXposLoop1
	STA RESBL	; Enable (or disable) missile 0 graphics
	RTS

; Kernel starts here. It's a 2-lines kernel that produces a single line resolution output.
; A kernel that draws a line of tiles and a kernel that draws a line with the ball are
; executed alternatively. After every row of tiles, we have 2 blank lines to load the new tiles'
; data and init stuff.
DrawPlayfield
	STA LoopCount	; 3	(40 cycles)
	LDA #0		; 2
	STA HMP0	; 3
	STA HMP1	; 3
	LDA #BGCOLOR	; Sprites color
X1:
	STA     WSYNC

	SLEEP 54	; Positioning 68 cycles on the screen

	; Conditioned branch to make kernel start alternatively with the sprites and with the ball lines.
	LDX ScanNumber	; 3
	BNE BgLine	; 2	= 76 if branch is taken, 75 if not
	JMP NormalLine	; 3	= 78

BgLine	; Start of the Background Line
	NOP		; 2
X3:
	STA COLUP0	; 3
	STA COLUP1	; 3
	LDY LoopCount	; 3
	LDA (ptr0),Y	; 5
	STA GRP0		; 3	17
	LDA (ptr1),Y	; 5
	STA GRP1		; 3
	LDA (ptr2),Y	; 5
	STA GRP0		; 3	33
	LDA (ptr5),Y	; 5
	STA Temp		; 3
	LDA (ptr4),Y	; 5
	TAX		; 2	48
	LDA (ptr3),Y	; 5
	LDY Temp		; 3	56
	STA GRP1		; 3
	STX GRP0		; 3
	STY GRP1		; 3
	STA GRP0		; 3	68

	DEC LoopCount		; 5
	BPL NormalLine		; 3 if branch is taken	76
	JMP NextRow		; 78	(must be 78) - Jump is taken if the row of tiles has been fully drawn

; Beginning of non-sprites line (another 76 cycles). Ball is drawn here.
NormalLine
	LDX #0			; 2
;NormalLine2
	STX COLUP0		; 3
	STX COLUP1		; 3	8
	STX ENABL	; 3

; 45 cycles
	DEC ScanCount	; 5
	STX HMOVE	; HMOVE must be hit around here...!
	LDA ScanCount,X	; 4 (add 1 cycle to make cycles count even)
	SEC		; 2
	SBC BallY	; 3
	ADC #BALL_HEIGHT; 2
	BCS DrawBall1	; 2/3
	SLEEP 24 -4
	STX ENABL	; 3
	JMP SkipDrawBall1	; 3	45
DrawBall1
	LDY BallTabPtr	; 3
	LDA BallSizeTab,Y	; 4
	STA CTRLPF	; 3
	LDA BallEnableShiftTab,Y	; 4
	STA ENABL	; 3
	STA HMBL	; 3
	DEC BallTabPtr	; 5	45
SkipDrawBall1

;	SLEEP 4-2
;	SLEEP 3

	LDY LoopCount	; 3
	LDA TilesColor,Y	; 4
	ADC LevelColor	; 3
	;LDA (PtrLo),Y	; 5

	DEC LoopCount	; 5
	; Should hmove ball or change size on odd or even lines only depending on which line we've started with.
	; (so that it results that we always change its values on the same line).
	; For this purpose, must use a set of 2 precalculated tables with HMOVEs and CTRLPFs with alternated
	; values.
;	STA HMBL
;	LDX LoopCount	; 3
	BPL     X3	; 3	= 76 if branch is taken back to X3, 75 if continuing. Draw next of the row of tiles
	JMP NextRow	; 78 (must be 78)

	org $fc00

SkipDrawBall2
	LDA #0		; 2
	STA ENABL	; 3	43
	SLEEP 16
	JMP EndDrawBall2	; 3

; We arrive here when a rof of tiles is fully drawn.
; We have 2 blank lines minus 2 cycles here for a total of 150 cycles
NextRow		; 76 cycles -2 from the jump to here
;	STA HMCLR
	DEC RowsNumber	; 5
	BEQ EndScreen	; 2
	LDA #0		; 2
;	NOP		; 2
	STA HMOVE	; 3
	STA COLUP0	; 3
	STA COLUP1	; 3

; --- Draw Ball ---
;	LDX #2		; 2
	DEC ScanCount	; 5
	LDA ScanCount	; 3
	SEC		; 2
	SBC BallY	; 3
	ADC #BALL_HEIGHT; 2
;	BCS DrawBall2	; 2/3
	BCC SkipDrawBall2	; 2/3
;	LDA #0		; 2
;	STA ENABL	; 3	43
;	SLEEP 18
;	JMP EndDrawBall2	; 3
DrawBall2
	LDY BallTabPtr	; 3
	LDA BallSizeTab,Y	; 4
	STA CTRLPF	; 3
	LDA BallEnableShiftTab,Y	; 4
;	STA ENABL	; 3
	STA HMBL	; 3
	DEC BallTabPtr	; 5	; 47
	STA ENABL	; 3	43
EndDrawBall2

; --- Load pointers with next row of level data (62 cycles)
	LDX LevelDataPtr	; 3
	CPX #1			; 2
	BMI LevelWrap		; 2/3
	JMP NoLevelWrap		; 3
LevelWrap
	LDX #LEVELRAM_LASTBYTE	; 2
NoLevelWrap
; Unrolling loop to save some cycles...
; ---
	LDA LevelRam,X	; 4
	STA ptr0	; 3
	DEX	; 2
	LDA LevelRam,X	; 4
	STA ptr1	; 3
	DEX	; 2
	LDA LevelRam,X	; 4
	STA ptr2	; 3
	DEX	; 2
	LDA LevelRam,X	; 4
	STA ptr3	; 3
	DEX	; 2
	LDA LevelRam,X	; 4
	STA ptr4	; 3
	DEX	; 2
	LDA LevelRam,X	; 4
	STA ptr5	; 3
	DEX	; 2
	STX LevelDataPtr	; 3

;	SLEEP 3
	LDY #7		; 2
	STY LoopCount	; 3

	LDA TilesColor,Y	; 4
	ADC LevelColor	; 3
;	LDA (PtrLo),Y	; 5
	LDX ScanNumber	; 3	Next line: tiles or ball?
	BNE BgLine2	; 2
	BIT $00		; 3
	JMP NormalLine	; 3	13
;	LDX AlwaysZero	; 2
;	JMP NormalLine2	; 3	13
BgLine2
	NOP		; 2+1
	JMP X3		; 3	13	76

; 36
EndScreen
	LDA #0
	STA COLUP0
	STA COLUP1
	STA ENABL
;	LDA #$6f
	STA COLUBK
	STA WSYNC
;	LDA     #0
	STA     GRP0
	STA     GRP1
;	STA     GRP0
;	STA     GRP1
;	STA ENAM0
;	STA ENAM1
;	STA ENABL
	RTS

; ------------------------------------------------
; ----------- David Crane's LeftRandom -----------
; ------ From Thomas Jentzsch's disassembly ------
; ------------------------------------------------

; *** Only requried for DEMO release

; Y: number to pick from the list (e.g. 0 = 1st number)
LeftRandom	subroutine
loopRandom:
; random' = random >> 1 | (bit4^bit5^bit6^bit1) * $80
	lda    random           ; 3
	asl                     ; 2
	eor    random           ; 3
	asl                     ; 2
	eor    random           ; 3
	asl                     ; 2
	asl                     ; 2
	rol                     ; 2
	eor    random           ; 3
	lsr                     ; 2
	ror    random           ; 5
;	dey                     ; 2
;	bpl    loopRandom      ; 2	Execute only once
	RTS

; Remove this sub for final game (only needed for DEMO release)
InitRandomLevel	subroutine
	JSR LeftRandom
	AND #3
	CLC
	ADC #1	; 1..4
;	LDA #$03
	STA CurrentLevel
	RTS

	org $fd00

BallShiftTab

BallEnableShiftTab
	.byte 0, 240, 242, 240
	.byte 0, 242, 2, 240
	.byte 2, 242, 2, 18
	.byte 2, 226, 2, 34


; 224
; 0
; 32
; <- 2
; 0
; -> 2

BallSizeTab
	.byte 5, 5, 5, 5	; very small
	.byte 5, 21, 21, 5	; small
	.byte 21, 37, 37, 21	; medium
	.byte 37, 53, 53, 37	; big

BallEnableTab

BallAnimTab
	.byte 0, 7, 11, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 11, 7, 3
	.byte 0, 3, 7, 11, 15, 15, 15, 15, 15, 11, 7

; 5  = 1 clock
; 21 = 2 clocks
; 37 = 4 clocks
; 53 = 8 clocks

TilesColor
	IF PAL_COLORS
	.byte $00, $d4, $d6, $d8, $d8, $d8, $d6, $d4	; PAL
	ELSE
	.byte $00, $04, $06, $08, $08, $08, $06, $04	; NTSC
	ENDIF

; 00 00 - Empty
; 01 08 - Normal
; 02 10 - Can't destroy
; 03 18 - Arrow Right	->
; 04 20 - Arrow Left	<-
; 05 28 - Arrow Down	v
; 06 30 - Arrow Up	^
; 07 38 - Double Jump

EatLines	subroutine
	STA WSYNC
	DEX
	BNE EatLines
	RTS

WaitINTIM	subroutine
.WaitTimer
	LDA INTIM
	BNE .WaitTimer
	RTS

LevelPointers
	.byte >LevelData1, <LevelData1, >LevelData2, <LevelData2, >LevelData3, <LevelData3, >LevelData4, <LevelData4

Number0
	.byte %01111100
	.byte %11000110
	.byte %11000110
	.byte %11000110
	.byte %11000110
	.byte %11000110
	.byte %11000110
	.byte %01111100
Number1
	.byte %11111100
	.byte %00110000
	.byte %00110000
	.byte %00110000
	.byte %00110000
	.byte %11110000
	.byte %01110000
	.byte %00110000
Number2
	.byte %11111110
	.byte %11000000
	.byte %01100000
	.byte %00110000
	.byte %00011000
	.byte %00001100
	.byte %11000110
	.byte %01111100
Number3
	.byte %01111100
	.byte %11000110
	.byte %00000110
	.byte %00000110
	.byte %00011100
	.byte %00000110
	.byte %11000110
	.byte %01111100
Number4
	.byte %00000110
	.byte %11111110
	.byte %11000110
	.byte %01100110
	.byte %01100110
	.byte %00110000
	.byte %00110000
	.byte %00011000
Number5
	.byte %01111100
	.byte %11000110
	.byte %00000110
	.byte %00000110
	.byte %11111100
	.byte %11000000
	.byte %11000000
	.byte %11111110
Number6
	.byte %01111100
	.byte %11000110
	.byte %11000110
	.byte %11000110
	.byte %11111100
	.byte %11000000
	.byte %11000110
	.byte %01111100
Number7
	.byte %00110000
	.byte %00110000
	.byte %00011000
	.byte %00011000
	.byte %00001100
	.byte %00001100
	.byte %00000110
	.byte %11111110
Number8
	.byte %01111100
	.byte %11000110
	.byte %11000110
	.byte %11000110
	.byte %01111100
	.byte %11000110
	.byte %11000110
	.byte %01111100
Number9
	.byte %01111100
	.byte %11000110
	.byte %00000110
	.byte %01111110
	.byte %11000110
	.byte %11000110
	.byte %11000110
	.byte %01111100

TilesLeftSprite
	.byte %00000000
	.byte %11111000
	.byte %11011010
	.byte %10101000
	.byte %11011010
	.byte %10101000
	.byte %11111000
	.byte %00000000

LevelSprite
	.byte %00000000
	.byte %00111000
	.byte %00100010
	.byte %00100000
	.byte %00100010
	.byte %00100000
	.byte %00000000
	.byte %00000000

LifeSprite
	.byte %00111000
	.byte %01111100
	.byte %01111100
	.byte %00111000

	org $fe00

BgTiles
BgTiles0
	; 0 Empty (Kills you)
	.byte %00000000
	.byte %00000000
	.byte %00000000
	.byte %00000000
	.byte %00000000
	.byte %00000000
	.byte %00000000
	.byte %00000000
BgTiles1
	; 1 Normal
	.byte %11111110
	.byte %11010110
	.byte %10101010
	.byte %11010110
	.byte %10101010
	.byte %11010110
	.byte %10101010
	.byte %11111110
BgTiles2
	; 2 Can't destroy
	.byte %11111110
	.byte %10111010
	.byte %11111110
	.byte %11111110
	.byte %11111110
	.byte %11111110
	.byte %10111010
	.byte %11111110
BgTiles3
	; 3 Arrow Right
	.byte %11111110
	.byte %11111110
	.byte %11110110
	.byte %10000010
	.byte %10000010
	.byte %11110110
	.byte %11111110
	.byte %11111110
BgTiles4
	; 4 Arrow Left
	.byte %11111110
	.byte %11111110
	.byte %11011110
	.byte %10000010
	.byte %10000010
	.byte %11011110
	.byte %11111110
	.byte %11111110
BgTiles5
	; 5 Arrow Down
	.byte %11111110
	.byte %11101110
	.byte %11000110
	.byte %11101110
	.byte %11101110
	.byte %11101110
	.byte %11101110
	.byte %11111110

	; 6 Arrow Up
	.byte %11111110
	.byte %11101110
	.byte %11101110
	.byte %11101110
	.byte %11101110
	.byte %11000110
	.byte %11101110
	.byte %11111110

	; 7 Double Jump
	.byte %11111110
	.byte %11000110
	.byte %10111010
	.byte %10101010
	.byte %10101010
	.byte %10111010
	.byte %11000110
	.byte %11111110

LogoSpr0
	.byte   %00111100
	.byte   %00110110
	.byte   %00110110
	.byte   %00111100
	.byte   %00110110
	.byte   %00110110
	.byte   %00111100
	.byte   %00000000
LogoSpr1
	.byte   %00000000
	.byte   %00111000
	.byte   %01101100
	.byte   %01101100
	.byte   %01101100
	.byte   %01101100
	.byte   %01101100
	.byte   %00111000
LogoSpr2
	.byte   %01110000
	.byte   %11011001
	.byte   %11011001
	.byte   %11011001
	.byte   %11011001
	.byte   %11011001
	.byte   %11011001
	.byte   %00000001
LogoSpr3
	.byte   %00000001
	.byte   %10110011
	.byte   %10110011
	.byte   %10110011
	.byte   %00110011
	.byte   %01110011
	.byte   %11010001
	.byte   %10010000
LogoSpr4
	.byte   %11000000
	.byte   %01100011
	.byte   %00000110
	.byte   %00000110
	.byte   %00000111
	.byte   %01100110
	.byte   %11000110
	.byte   %00000011
LogoSpr5
	.byte   %00001100
	.byte   %10000000
	.byte   %11001100
	.byte   %00001100
	.byte   %00001100
	.byte   %00001100
	.byte   %11001100
	.byte   %10000000

	org $ff00

; Level length in bytes: 6*height + 1
; Levels are reversed

; Level header description:
	; 1 Level color
	; 2 Tiles to destroy (BCD). Put $ anyway.
	; 3 starting location offset on the 78-bytes map in ram as 71-(X+Y*6)
	; 4 Ball start X (in pixels) as (x position on tiles)*8+63
	; 5 Ball start Y (in pixles) as 186-(y position on tiles)*5

LevelData
LevelData1
	; Level 1
	.byte $80, $16, 71-(0 + 0*6), 0*8+63, 186-0*5

	.byte $41,$55,$13
	.byte $41,$11,$13
	.byte $01,$00,$10
	.byte $41,$11,$13
	.byte $41,$11,$13
	.byte $12,$77,$21
	.byte $12,$77,$21
	.byte $41,$11,$13
	.byte $41,$11,$13
	.byte $01,$00,$10
	.byte $41,$11,$13
	.byte $41,$66,$12

LevelData2
	; Level 2
	.byte $10, $01, 71-(3 + 6*6), 3*8+63, 186-6*5

	.byte $46,$75,$21
	.byte $53,$56,$44
	.byte $44,$54,$55
	.byte $56,$45,$45
	.byte $53,$53,$53
	.byte $63,$24,$65
	.byte $67,$66,$65
	.byte $35,$73,$65
	.byte $46,$34,$45
	.byte $57,$64,$63
	.byte $45,$45,$46
	.byte $54,$73,$33

LevelData3
	; Level 3
	.byte $20, $22, 71-(2 + 5*6), 2*8+63, 186-5*5

	.byte $00,$00,$00
	.byte $00,$00,$00
	.byte $00,$11,$00
	.byte $01,$11,$10
	.byte $11,$00,$11
	.byte $10,$00,$01
	.byte $11,$22,$11
	.byte $10,$11,$01
	.byte $01,$11,$10
	.byte $00,$11,$00
	.byte $00,$00,$00
	.byte $00,$00,$00

LevelData4
	; Level 4
	.byte $b0, $31, 71-(0 + 0*6), 0*8+63, 186-0*5

	.byte $01,$17,$07
	.byte $11,$00,$00
	.byte $10,$00,$01
	.byte $11,$11,$10
	.byte $01,$10,$11
	.byte $00,$00,$11
	.byte $01,$11,$10
	.byte $11,$00,$00
	.byte $11,$07,$00
	.byte $01,$11,$00
	.byte $00,$01,$11
	.byte $00,$00,$02

;	.byte $01,$17,$01
;	.byte $11,$00,$00
;	.byte $11,$11,$10
;	.byte $01,$10,$11
;	.byte $00,$00,$11
;	.byte $00,$01,$10
;	.byte $01,$11,$00
;	.byte $11,$00,$00
;	.byte $11,$07,$00
;	.byte $01,$11,$00
;	.byte $00,$01,$11
;	.byte $00,$00,$02

;	.byte $00,$00,$00
;	.byte $00,$70,$10
;	.byte $07,$07,$70
;	.byte $00,$70,$00
;	.byte $00,$07,$00
;	.byte $00,$70,$00
;	.byte $00,$07,$07
;	.byte $70,$70,$00
;	.byte $00,$00,$07
;	.byte $70,$00,$00
;	.byte $00,$00,$07
;	.byte $70,$70,$72

	org $FFFC
	.word Start
	.word Start
